-- Espionage Overview Master Process File
-- Author: sman
-- DateCreated: 01/30/18 5:06:34 PM
--------------------------------------------------------------


include("IconSupport")
include("InstanceManager")

include("00_Globals.lua")
include("05_Instantiate_Globals.lua")														-- Instantiates date from the global tables into working data stores
--include("60_Mission_Support_Functions.lua")													-- Various support functions used by all of the following include files/this file
include("20_Process_Mission_Results.lua")													-- Process the results of a CO mission
include("30_Process_International_Responses.lua")											-- Determines international community responses for all covert operations
include("50_Process_Recon_Results.lua")														-- Builds the Recon Report popup/notification for the human player only; AI doesn't use Recon mission
include("25_Process_AI_Mission_Results.lua")												-- Performs AI missions, in a similar way that the human player does, without UI features


local g_ItemManager = InstanceManager:new( "COMenuInstance", "COMenuButton", Controls.COMenuStack )
local g_sSelectedAgentName
local g_iSelectedAgentRank
local g_pSelectedCity
local g_iSelectedMission

-- ===============================================================================================
--		Support Functions

-- ===============================================================================================
--		Covert Ops Mission Selection Popup 

function DisplayCOMenuPopup(sSelectedAgentName, iSelectedAgentRank, pSelectedCity, g_iMissionCost)		-- Function that builds the popup and displays it	
	if (not sSelectedAgentName) or (not pSelectedCity) then 
		print("Inside DisplayCOMenuPopup. No agent name or no city pointer error!")
		return
	end
	
	local pPlayer = Players[Game.GetActivePlayer()]											-- Get current player; ok to use GetActivePlayer as only the human player can use this popup	
	local iCurrentEra = pPlayer:GetCurrentEra()												-- Get Active (human) player's current Era
	local iAttackingPlayerGold = pPlayer:GetGold()											-- Get total gold target civ has on hand	
	local pTargetPlayer = Players[pSelectedCity:GetOwner()]									-- Get the target city owner's pointer
	
	
	g_pSelectedCity = pSelectedCity															-- Assign value to target city global pointer
	g_sSelectedAgentName = sSelectedAgentName												-- Assign value to global spy name
	g_iSelectedAgentRank = iSelectedAgentRank												-- Assign value to global spy rank (1..3)
	g_ItemManager:ResetInstances()
		
	local tValidMissionTable = BuildMissionMenu(pPlayer, g_SelectedCity)					-- Get number of "valid" missions for this player against this city; function should always return 1 (ABORT); number should be higher to display the menu
	if #tValidMissionTable < 2 then															-- Must be at least 1 "valid" mission, otherwise exit early
		print("DisplayCOMenuPopup. Attacker: " .. pPlayer:GetCivilizationDescription() .. "   Target City: " .. pSelectedCity:GetName() .. " - no valid missions available")
		return
	end
			
	local sTitleString = Locale.ConvertTextKey("TXT_KEY_COVERT_OPS_BODY", g_sSelectedAgentName, g_pSelectedCity:GetName())
	Controls.COMenuMessage:LocalizeAndSetText(sTitleString)

	local iAttackerEP = CalculateEspionagePosture(pPlayer, g_pSelectedCity)
	local iDefenderEP = CalculateEspionagePosture(pTargetPlayer, g_pSelectedCity)
	local sTgtAdjective = pTargetPlayer:GetCivilizationAdjective() or "Enemy"

	local sTitleString = Locale.ConvertTextKey("TXT_KEY_COVERT_OPS_BODY_TOOLTIP", g_pSelectedCity:GetName(), iAttackerEP, sTgtAdjective, iDefenderEP)
	--print("STitleString: " .. sTitleString)
	Controls.COMenuMessage:SetToolTipString(sTitleString)

	local iReliabiltyIDX = 1																-- Use variability in estimating success/incident probabilities (20190406)
	if (iAttackerEP - iDefenderEP) >= 10 then iReliabiltyIDX = 2 end						-- Lower variability as attacker EP improves
	if (iAttackerEP - iDefenderEP) >= 20 then iReliabiltyIDX = 3 end
	if (iAttackerEP - iDefenderEP) >= 30 then iReliabiltyIDX = 4 end
	
	for iNum = 1, #tValidMissionTable do													-- Cycle through all valid missions		
	
		local controlTable = g_ItemManager:GetInstance()		

		--IconHookup( tValidMissionTable[iNum] + 23, 64, "SMAN_COVERT_OPS_ACTION_ATLAS", controlTable.Icon64 )	-- Silver Icon at top of popup
		IconHookup( tValidMissionTable[iNum] - 1, 64, "SMAN_COVERT_OPS_ACTION_ATLAS", controlTable.Icon64 )	-- Gold Icon at top of popup

		local keyMissionTitleString = "TXT_KEY_COVERT_OPS_MISSION_TITLE_" .. tValidMissionTable[iNum]		-- Get the correct text strings for this mission type
		local keyMissionBodyString = "TXT_KEY_COVERT_OPS_MISSION_BODY_" .. tValidMissionTable[iNum]
		
		controlTable.COMenuLabel:LocalizeAndSetText(keyMissionTitleString)
		
		iMissionSuccess = AdjustCount(g_MissionList[tValidMissionTable[iNum]][ML_MISSION_SUCCESS], iReliabiltyIDX, true)	-- Adjust estimate based on attacker EP
		iResponseChance = AdjustCount(g_MissionList[tValidMissionTable[iNum]][ML_CHANCE_INCIDENT], iReliabiltyIDX, true)	-- Adjust estimate based on attacker EP
		g_iMissionCost = GetMissionCost(pPlayer, tValidMissionTable[iNum], pTargetPlayer, pSelectedCity)
		
		iMissionSuccess = SetBounds(iMissionSuccess, 5, 95)
		iResponseChance = SetBounds(iResponseChance, 5, 95)

		controlTable.COMenuButton:SetToolTipString(Locale.ConvertTextKey(keyMissionBodyString, iMissionSuccess, iResponseChance, g_iMissionCost))
		controlTable.COMenuButton:SetDisabled(false)	

		local selectionAnim = controlTable.COMenuSelectionAnim
		controlTable.COMenuButton:SetVoid1(tValidMissionTable[iNum])
		controlTable.COMenuButton:RegisterCallback(Mouse.eLClick, function()  g_iSelectedMission = tValidMissionTable[iNum]  DisplayCOMenuPopup(sSelectedAgentName, iSelectedAgentRank, g_pSelectedCity, g_iMissionCost)  end)		

		if g_iSelectedMission then
			if g_iSelectedMission == tValidMissionTable[iNum] then
					controlTable.COMenuSelectionAnim:SetHide(false)
			else
					controlTable.COMenuSelectionAnim:SetHide(true)
			end
		else
				controlTable.COMenuSelectionAnim:SetHide(true)
		end
	end	
	
	Controls.COMenuStack:CalculateSize()
	Controls.COMenuStack:ReprocessAnchoring()
	Controls.COMenuPanel:CalculateInternalSize()	
	--Controls.COMenuPanel:SetScrollValue(0)
	Controls.ConfirmButton:SetDisabled(true)
	Controls.COMenuPopup:SetHide(false)

	if g_iSelectedMission then
		Controls.ConfirmButton:SetDisabled(false)
	end	
end
LuaEvents.CovertMissionPopup.Add( DisplayCOMenuPopup )


function OnConfirm(playerID)																-- Process actions once the player has selected a mission	
	g_iActiveMission = g_iSelectedMission
	g_iSelectedMission = ABORT_MISSION														-- Sets the default menu position to "Abort Mission" for the next time the popup unhides
	Controls.COMenuPanel:SetScrollValue(0)
	Controls.COMenuPopup:SetHide(true)
	
	--print("========> Selected mission: " .. g_iActiveMission .. "   Mission name: " .. g_MissionList[g_iActiveMission][ML_MISSION_NAME])

	-- =====================		Special Handling Instructions for Mission: Destroy (normal) Building		===================
	if g_iActiveMission == DESTROY_BUILDING then											-- Special process for destroy building - needs unique popup
		LuaEvents.ProcessSelectBuilding(g_SelectedCity)	
	else
		ProcessMission (g_iActiveMission, g_pSelectedCity, -1)								-- Process the selected mission normally; -1=Selected Building						
	end	   	

	if g_iActiveMission == ABORT_MISSION then
		Controls.LaunchCOButton:SetDisabled(false)											-- Disable the launch button
	else
		Controls.LaunchCOButton:SetDisabled(true)											-- Disable the launch button		
	end	
end
Controls.ConfirmButton:RegisterCallback( Mouse.eLClick, OnConfirm )


-- ====================================================================================================================================
--		Recon/Mission results popup

local g_ReconReportManager = InstanceManager:new("ReconReport", "ReconReportLabel", Controls.ReconReportStack)

function ProcessReconPopup (sReportTitle, sReportHeader, sReconReport, iTopIcon)	
	
	g_ReconReportManager:ResetInstances()
		
	IconHookup( iTopIcon + 57, 80, "SMAN_COVERT_OPS_ACTION_ATLAS", Controls.ReconReportTopIcon )	-- Icon at top of popup, 1=gold, 2=silve, 3=red
		
	Controls.ReconReportTitle:LocalizeAndSetText("TXT_KEY_COVERT_OPS_RECON_REPORT_TITLE", sReportTitle)		-- Report title (green bar)
	Controls.ReconReportHeader:LocalizeAndSetText("TXT_KEY_COVERT_OPS_RECON_REPORT_HEADER", sReportHeader)		-- Set value for header of report
	
	local instance = g_ReconReportManager:GetInstance()
	instance.ReconReportLabel:SetText(sReconReport)											-- Set value for body of report

	Controls.ReconReportStack:CalculateSize()
	Controls.ReconReportStack:ReprocessAnchoring()
	Controls.ReconReportPanel:CalculateInternalSize()
	Controls.ReconReportPanel:SetScrollValue(0)
		
	Controls.ReconReportPopup:SetHide(false)
end
LuaEvents.ProcessReconReport.Add( ProcessReconPopup )


function OnReconReportOK()
	Controls.ReconReportPopup:SetHide(true)
end
Controls.ReconReportOK:RegisterCallback(Mouse.eLClick, OnReconReportOK)


-- ====================================================================================================================================
--		Select Building popup - returns a building ID or -1 (random building)

local g_SortTable																			-- Global table used in sort function

function OnCOSBOK()																			-- "Confirm" button pressed, building selection is final
	Controls.COSBPopup:SetHide(true)														-- Close the popup
	if (not g_SelectedBuildingID) or (g_SelectedBuildingID < 0) then						-- If there's ANY problem with the selected building, return "random building"
		g_SelectedBuildingID = -1															-- Random building "ID"		
	end
	--print("Exiting menu.  Selected building ID: " .. g_SelectedBuildingID)					-- Display the selected building ID
	ProcessMission (g_iActiveMission, g_pSelectedCity, g_SelectedBuildingID)				-- Process the selected mission	
end
Controls.COSBOK:RegisterCallback(Mouse.eLClick, OnCOSBOK)									-- Register the callback

function SortByName(a, b)																	-- QUAD sort function
	local sNameA = g_SortTable[tostring(a)].Name
	local sNameB = g_SortTable[tostring(b)].Name
	return sNameA < sNameB
end

function OnBuildingSelected(iBuildingID)													-- "Building has been selected" callback - "selected" but not confirmed!
	local sBuildingName = "Random Building"  												-- Default selected building name is "Random"
	if not iBuildingID then iBuildingID = -1 end											-- If not a good ID, then force it to -1 (Random)
	if iBuildingID ~= -1 then																-- If the building ID isn't random, get the actual building name
		sBuildingName = Locale.ConvertTextKey(GameInfo.Buildings[iBuildingID].Description)	-- Localize the string
	end	 
	Controls.COSBMenu:GetButton():SetText(sBuildingName)									-- Set the "button" to the building name so it displays properly
	local sAtlas = "CIV_COLOR_ATLAS"														-- This atlas/portrait combo is for the "Question Mark" icon
	local iPortraitIDX = 23
	if iBuildingID ~= -1 then																-- If the building isn't random, get the actual icon for the building
		sAtlas = GameInfo.Buildings[iBuildingID].IconAtlas
		iPortraitIDX = GameInfo.Buildings[iBuildingID].PortraitIndex
	end
	IconHookup( iPortraitIDX, 128, sAtlas, Controls.SelectBuildingCenterIcon )				-- Display the building/question mark in the center of the popup			
	--print("Selected building name: " .. sBuildingName .. "    ID: " .. iBuildingID .. "    Atlas: " .. sAtlas .. "    Portrait Index: " .. iPortraitIDX)
	g_SelectedBuildingID = iBuildingID														-- Set the PROPOSED return value to the currently selected building	
end

function UpdateBuildingList(pTargetCity)													-- Construct a list of non-wonder buildings in pTargetCity
	Controls.COSBMenu:ClearEntries()														-- Clear any previous values that might have been there
	g_SortTable = {}																		-- Clear the SortTable
	local sCOSBTitle = Locale.ConvertTextKey("TXT_KEY_CO_COSB_MESSAGE", pTargetCity:GetName())	-- Update the "Message" on the popup (e.g. "Attack in Omaha")
	Controls.COSBMessage:LocalizeAndSetText(sCOSBTitle)										-- Set the title (inside top green bar) for the popup
	IconHookup(23, 128, "CIV_COLOR_ATLAS", Controls.SelectBuildingCenterIcon)				-- Initial center icon will be the question mark
	local tEntry = {}																		-- Create an entry in the Pulldown Stack
	Controls.COSBMenu:BuildEntry("InstanceOne", tEntry)									
	g_SortTable[tostring(tEntry.Button)] = {Name="Random Building"}							-- Add "Random Building" to SortTable
	tEntry.Button:SetVoid1(-1)																-- Void1 on this entry
	tEntry.Button:SetText("Random Building")												-- Set the text value for the menu item
	for row in GameInfo.Buildings() do														-- Scroll through all buildings in the Buildings table
		if pTargetCity:GetNumRealBuilding(row.ID) ~= 0  then								-- If the city has more than zero of these building types, then...
			local BClass = GameInfo.Buildings[row.ID].BuildingClass							-- Get the Building class of the building in the city
			local BWonder = GameInfo.BuildingClasses["" .. BClass .. ""].MaxGlobalInstances		-- Get MGI amount; normal buildings = 0; global wonders ~= 0
			local BNational = GameInfo.BuildingClasses["" .. BClass .. ""].MaxPlayerInstances	-- Get MPI amount; normal buildings = 0; national wonders ~= 0
			if (BWonder == -1) and (BNational == -1) then									-- If not a Global or National wonder then...
				local sBuildingName = Locale.ConvertTextKey(row.Description)				-- Get the name of this building
				local entry = {}															-- Create an entry in the Pulldown Stacl
				Controls.COSBMenu:BuildEntry("InstanceOne", entry)						
				g_SortTable[tostring(entry.Button)] = {Name=sBuildingName}					-- Add this building name to the SortTable in the right location
				entry.Button:SetVoid1(row.ID)												-- Void1 on this entry
				entry.Button:SetText(sBuildingName)											-- Set the text value for the menu item
			end
		end
	end	
	Controls.COSBMenuStack:SortChildren(SortByName)											-- Sort items in the menu stack
	Controls.COSBMenu:GetButton():LocalizeAndSetText("TXT_KEY_CO_COSB_MENU_CHOOSE")			-- Prompt string for the pulldown menu
	Controls.COSBMenu:CalculateInternals()													-- Popup bookkeeping
	Controls.COSBMenu:RegisterSelectionCallback(OnBuildingSelected)							-- Register the OnBuildingSelected callback
end

function ProcessSelectBuildingPopup(pTargetCity)											-- Return a building ID or -1 for Random Building	
	g_SelectedBuildingID = -2	
	UpdateBuildingList(pTargetCity)															-- Get a list of all non-wonder buildings in pTargetCity
	Controls.COSBPopup:SetHide(false)														-- Display the building list popup
end
LuaEvents.ProcessSelectBuilding.Add( ProcessSelectBuildingPopup )

