-- Process_AI_Mission_Results.lua
-- Author: sman
-- DateCreated: 01/11/18 4:06:05 AM
--------------------------------------------------------------

include("40_Process_INTREPs.lua")															-- Processes RUMINTs and FIRs

-- =================================================================================
--			Globals

gAiMissionTable = {}																		-- Master table holding all AI covert missions - both open and closed
	
-- =================================================================================
--			Functions

function OpenMissionStats (pPlayer)															-- Determine how many open missions pPlayer has, and what game turn the most recent mission is supposed to occur
	local iMissionCount = 0
	local pPlayerID = pPlayer:GetID()	
	if #gAiMissionTable > 0 then															-- If there are any missions in the table, then...
		for iIndex = 1, #gAiMissionTable do													-- Cycle through all the missions
			if (gAiMissionTable[iIndex][1] == pPlayerID) and (gAiMissionTable[iIndex][2] == YES) then		-- Does this mission belong to pPlayerID [1] and is it still open [2]?
				iMissionCount = iMissionCount + 1											-- Bump the counter for pPlayer				
			end
		end
	end
	--print ("Total missions: " .. #gAiMissionTable .. "    total missions for player " .. pPlayerID .. ":  " .. iMissionCount)
	return iMissionCount																	-- Return total # of open missions for pPlayer
end


function GetTargetPlayer(pPlayer)															-- Select a target civ for this operation; return target ID or -1 if no valid targets
	local ipPlayerID = pPlayer:GetID()														-- Get attacking player's ID	
	local iTargetPlayerID = -1																-- Evaluates a set of fixed criteria to determine the relationship between the attacking civ and all other civs (human too)
	local iPlayerTeam = pPlayer:GetTeam()
	local pPlayerTeam = Teams[iPlayerTeam]													-- Then adjusts those criteria by a random +- 25%.  Civ with the "worst" relationship is the target
	local iWorstEnemyCount = 0	
	--print("Getting a target player for player: " .. pPlayer:GetID())
	local iChance = math.random(1, 100)
	if iChance < 10 then																	-- Get a truly random civ to attack (10% of the time)
		local tLivingCivs = {}
		for iPlayer=0, GameDefines.MAX_CIV_PLAYERS-1 do										-- Cycle through all players, excluding barbs
			local pTempTeam = Teams[Players[iPlayer]:GetTeam()]								-- Get team pointer for loop player

			if pTempTeam:IsHasMet(pPlayer:GetTeam()) and									-- Only consider civs we've met
					(Players[iPlayer]:IsAlive()) and										-- Only consider living civs
					(not Players[iPlayer]:IsBarbarian()) and								-- Don't consider barbs
					(iPlayer ~= ipPlayerID) then											-- Only consider civs who are not ourselves		
						
				local iPlayerScore = Game.GetPlayerScore(iPlayer) or 0						-- CHANGE: added highest game score as a potential determinant for target (sman:20180209)				
				tLivingCivs[#tLivingCivs+1] = { [1] = Players[iPlayer]:GetID(), [2] = iPlayerScore }	-- Add this player ID to the table of possible targets and current score				
			end
		end
		if #tLivingCivs > 0 then															-- If there are 1 or more potential targets, then
			if math.random(1, 100) < 50 then												-- 50:50 chance the target will be truly random, or based on highest game score of known civs;  true random is a third of a third, or 11% actual total chance (sman:20180209)
				iTargetPlayerID = tLivingCivs[math.random(1, #tLivingCivs)][1]				-- Worst enemy is an equally random/living/met major civ
			else																			-- New process for targeting known civ with highest score (sman:20180209)
				iHiScoreIDX = 1																-- Assume 1st Civ in table is hi score
				for iDX = 1, #tLivingCivs do												-- Scroll through all civs in table (all known civs)
					if tLivingCivs[iDX][2] > tLivingCivs[iHiScoreIDX][2] then				-- If new civ score is higher, then
						iHiScoreIDX = iDX													-- Make this civ the new Hi scorer/target
					end
				end
				iTargetPlayerID = tLivingCivs[iHiScoreIDX][1]								-- Make the actual taget civ the highest scorer we found				
			end				
		else																				-- Any issues means the function will return -1 and no enemy is selected
			iTargetPlayerID = -1
		end

	-- ===========================================================================================================================================================
	--		Version 5 change - add a chance that the target of a Covert Op will be the game's highest scorer - if it that civ is known to the AI and they don't really like him....
	--elseif iChance < 33 then																-- Go after highest game score who's not an 'ally' (23% of the time)
	elseif iChance < 93 then																-- Go after highest game score who's not an 'ally' (23% of the time)
		local iTargetCivID = -1
		local iTargetHighScore = -1

		for iDX = 0, GameDefines.MAX_MAJOR_CIVS-1 do										-- Cycle through all players
			if iDX ~= ipPlayerID then														-- If the loop player isn't pPlayer, then...
				local pLoopPlayer = Players[iDX]
				if pLoopPlayer:IsAlive() then												-- If loop player is alive, then...
					local pLoopTeam = Teams[pLoopPlayer:GetTeam()]							-- Get team pointer for loop player
					if ( pLoopTeam:IsHasMet(pPlayer:GetTeam()) ) and						-- If the loop player has met pPlayer, AND...
						(iPlayerTeam ~= pLoopPlayer:GetTeam())	and							-- They are not on the same team, AND...
						(not pPlayer:IsDoF(iDX) ) then										-- pPlayer does not have a DoF with loop player, then...

						local iLoopPlayerScore = pLoopPlayer:GetScore()						-- Get the game score of the loop player
						if iLoopPlayerScore > iTargetHighScore then							-- If that score is more than the previous high score, then...
							iTargetCivID =  iDX												-- Make loop player the highest score...  So far...
							iTargetHighScore = iLoopPlayerScore
						end
					end
				end
			end
		end

		if iTargetCivID > -1 then															-- We found at least 1 Civ who met the criteria above...
			--print("High score tgt.  Civ: " .. iTargetCivID .. "     Score: " .. iTargetHighScore)
			iTargetPlayerID = iTargetCivID
		--else
			--print("High score tgt - no civs fit the description...")
		end
	-- ===========================================================================================================================================================


	else																					-- Use a formula/process to determine worst enemy
		
		local iLoyaltyAdjustment = (5 - g_Leader_Stats[ipPlayerID][LOYALTY]) * 3			-- Allows attacking leader's LOYALTY stat to impact attacking a target civ with a DOF in place (sman:20180220)

		for iPlayer=0, GameDefines.MAX_MAJOR_CIVS-1 do										-- Cycle through all players, will eliminate barbs later			
			
			local iTempTeam = Players[iPlayer]:GetTeam()									-- Some methods need target team ID instead of target player ID
			local pTempTeam = Teams[iTempTeam]												-- Get team pointer for loop player
			
			if pTempTeam:IsHasMet(iPlayerTeam) and											-- Only consider civs we've met
					(Players[iPlayer]:IsAlive()) and										-- Only consider living civs
					--(not Players[iPlayer]:IsBarbarian()) and								-- Don't consider barbs
					(iPlayerTeam ~= iTempTeam) and											-- Don't let team members conduct Covert Ops against each other
					(iPlayer ~= ipPlayerID) then											-- Only consider civs who are not ourselves				
				
				local iBadnessCount = 0														-- Badness is a summary of all the things the target civ has done against pPlayer			

				if pPlayer:IsCapitalCapturedBy(Players[iPlayer]:GetID()) then				-- They have our capital...
					iBadnessCount = iBadnessCount + 20
				end

				if (pPlayer:IsNukedBy(Players[iPlayer]:GetID())) then						-- The nuked us in the past...
					iBadnessCount = iBadnessCount + 10
				end

				if pPlayerTeam:IsAtWar(iTempTeam) then										-- We are at war with them...
					iBadnessCount = iBadnessCount + 10
				end

				if Players[iPlayer]:IsDenouncingPlayer(ipPlayerID) then						-- They are denouncing us...
					iBadnessCount = iBadnessCount + 7
				end

				if pPlayer:IsPlayerRecklessExpander(Players[iPlayer]:GetID()) then			-- They are reckless expanders...
					iBadnessCount = iBadnessCount + 5
				end

				if Players[iPlayer]:IsHasPlayerBulliedProtectedMinor() then					-- They have bullied protected city states...
					iBadnessCount = iBadnessCount + 2
				end

				if Players[iPlayer]:IsEmpireUnhappy() then									-- They are unhappy...
					iBadnessCount = iBadnessCount + 1
				end
				if Players[iPlayer]:IsEmpireVeryUnhappy() then								-- They are very unhappy...
					iBadnessCount = iBadnessCount + 2
				end
				if Players[iPlayer]:IsEmpireSuperUnhappy() then								-- They are super unhappy...
					iBadnessCount = iBadnessCount + 4
				end
			
				local iPlayerPower = pPlayer:GetPower()										-- Evaluate relative strength.  Get pPlayer's "power"
				local iTargetPlayerPower = Players[iPlayer]:GetPower()						-- Get Target player's "power"
				local iPowerRatio = math.floor(iTargetPlayerPower / iPlayerPower)			-- Get the ratio of strength
				if iPowerRatio < 1 then iPowerRatio = math.floor(iPlayerPower / iTargetPlayerPower) * -1 end	-- If enemy is stronger, then reverse the ratio and make it negative
				if iPowerRatio > 3 then iPowerRatio = 3 end									-- After 3 times larger, it doesn't matter
				if iPowerRatio < -3 then iPowerRatio = -3 end								-- After 3 times smaller, it doesn't matter

				iBadnessCount = iBadnessCount + (iPowerRatio * 4)							-- Change iBadnessCount: if we are stronger, add the Ratio*4; if they are stronger, subtract the reversered ration by Ratio*4; max is +-12 points

				local iAdjustment = (math.random(1, 50) - 25) 								-- Add some randomness to the overall equation (+- 25%)			
				--print("Target civ: " .. iPlayer .. "   ibad: " .. iBadnessCount .. "    Adjustment: " .. iAdjustment)			
				iBadnessCount = math.floor (iBadnessCount * (iAdjustment/100) )				-- Adjust the overall badness bt +-25; provides some variety - especially for really "bad" enemies			
				
				if pPlayer:IsDoF(iPlayer) then												-- If iPlayer is DoF with pPlayer, cut chance in half (sman:20180213)
					iBadnessCount = math.floor (iBadnessCount / 2 )	
					iBadnessCount = iBadnessCount + iLoyaltyAdjustment						-- Adjust by attacking leader Loyalty value; most are 3..7; 5 is baseline - higher than that subtracts from likelihood of attacking; lower increases likliehod (sman:20180220)
					--print("Cutting Badness due to DoF between players " .. pPlayer:GetID() .. " and " .. iPlayer .. "    Loyalty Adjustment: " .. iLoyaltyAdjustment)
				end				
				
				if iBadnessCount > iWorstEnemyCount then									-- if iPlayer's badness is worse than our previous "worst enemy" then...
					iTargetPlayerID = Players[iPlayer]:GetID()								-- We have a new worst enemy
					iWorstEnemyCount = iBadnessCount
				end
			end
		end
	end
	--print("Worst enemy/target: " .. iTargetPlayerID)	
	return iTargetPlayerID																	-- Returns -1 if no civs can be attacked - not sure how this could happen, though
end


function Component1_Schedule_Mission(pPlayer, iGameTurn, iCurrentEra)						-- Determines if pPlayer can schedule a mission this turn
	if iCurrentEra < ERA_CLASSICAL_ID then return end										-- No missions allowed until in Classical Era (sman:20180211)

	if math.random(1, 100) > g_iAI_ChanceToLaunch then return end							-- If random percent > AI chance to launch, exit early

	local ipPlayerID = pPlayer:GetID()														-- Gets the player ID of pPlayer
	local iOpenMissionCount = OpenMissionStats (pPlayer)									-- Get # of open missions for this player
	
	if iOpenMissionCount >= (iCurrentEra + 1) then return end								-- Can never have more open missions than the Era+1 you're in
	if iOpenMissionCount >= g_Open_Mission_Limit then return end							-- Only let pPlayer have g_Open_Mission_Limit open missions at a time, which is adjusted for difficulty/gamespeed
				
	local iToAttackOrNotToAttack = math.random(1, 100)										-- Chance of scheduling an Attack
		
	if ipPlayerID == 0 then ipPlayerID = 99 end												-- Set index to 99 for the human player - should only hit during Live Tuner games
	if iToAttackOrNotToAttack > g_Leader_Stats[ipPlayerID][COVERT_OPERATIONS_PROPENSITY] then return end		-- If random % is greater than COP threashold, exit early; otherwise SCHEDULE!
			
	local iTargetPlayerID = GetTargetPlayer(pPlayer)										-- Determine Target Civ - Who do we hate the most?
	if iTargetPlayerID == -1 then return end												-- If no valid civs to attack, exit early
			
	local iTurnToExecute = iGameTurn + math.random(8, 12)									-- Determine the actual turn to launch the mission			
	local iTurnToWarn = iTurnToExecute - math.random(0, 5)									-- Higher bound here needs to be 1 less than the lower bound of iTurnToExecute!
	if iTurnToWarn == iTurnToExecute then iTurnToWarn = -1 end								-- Sometimes, you don't get a warning...
			
	local iMissionNumber, iMissionCost = GetMostLikelyAIMission (pPlayer, iCurrentEra, iTargetPlayerID)	-- Will return a wighted, valid mission between 1..14, depending on pPlayer Era and affordable
	
	local bIsCapitalOK = true																-- Default to let capitals be valid targets for covert ops
	if iMissionNumber == STEAL_CITY then bIsCapitalOK = false end							-- If the mission is Steal City, then the capital is not a valid target
	
	local pTargetCity = GetRandomCity(pPlayer, Players[iTargetPlayerID], bIsCapitalOK)		-- Arguments: pPlayer, pTargetPlayer, bIsCapitalOK
	if not pTargetCity then return end														-- If nil, no available cities (e.g. steal city and target civ has only capital?) - return early
	

	if iMissionNumber ~= -1 then															-- Mission num should be 2 or higher; otherwise, AI has no valid missions and failed to schedule this opportunity, timer still starts over
		-- gAiMissionTable[iIndex][1] = attacking player ID    [2] = Is the mission open?  Will change to NO once the mission is completed    [3] = What turn was mission scheduled?
		--		[4] = What turn to warn?    [5] = What turn to execute?   [6] = What mission type will we launch?    [7] = Who is the target player(ID)?    
		--		[8] = Target City pointer   [9] = Placeholder element - used to store mission results after the mission - for future reference, [10] = Mission Cost, [11]=International response 
		gAiMissionTable[#gAiMissionTable + 1] = { [1] = pPlayer:GetID(), [2] = YES, [3] = iGameTurn, [4] = iTurnToWarn, [5] = iTurnToExecute, [6] = iMissionNumber, [7] = iTargetPlayerID, [8] = pTargetCity, [9] = "", [10] = iMissionCost, [11] = ""}
		--print("Scheduling a mission. Index: " .. iIndex .. "    Player: " .. pPlayer:GetID() .. "  on game turn: " .. iGameTurn .. "    warning turn: " .. iTurnToWarn .. "  planned turn: " .. iTurnToExecute .. "    target civ: " .. iTargetPlayerID ..  "    target city: " .. pTargetCity:GetName() .. "    iMissionNumber: " .. iMissionNumber .. "    iMissionCost: " .. iMissionCost)
	end

end


function Component2_Warn_Mission(pPlayer, iGameTurn, iCurrentEra)							-- Determines if a Warning Notification is required this turn - only for human player!	
	-- Function is kind of a placeholder for now - may expand it later
	ProcessFIR(pPlayer, iGameTurn, iCurrentEra)												-- Determine if a FIR will notify
	ProcessRUMINT(pPlayer, iGameTurn, iCurrentEra)											-- Determine if a RUMINT will notify
end


function Component3_Execute_Mission(pPlayer, iGameTurn, iCurrentEra)						-- Actually executes a previously-scheduled mission
	local pUnit = nil																		-- AI doesn't use units to launch Covert Ops
		
	for iIndex = 1, #gAiMissionTable do													-- Cycle through all missions

		if (gAiMissionTable[iIndex][1] == pPlayer:GetID()) and (gAiMissionTable[iIndex][5] == iGameTurn) and (gAiMissionTable[iIndex][2] == YES) and (pPlayer:IsAlive()) then		-- pPlayer:GetID() has a mission scheduled for this turn, if the mission is OPEN, and is still alive?  

			-- Mission results:		    Notify Type	FadeStr	Mission Result	Reaction	Msn		Incident	Summary		UnitSpawnType	Notify X,		Notify Y	---- default values for mission results data table
			--							1			2		3				4			5		6			7			8,				9,				10
			local tMissionResultData = { 1,			"",		"",				"",			1,		1,			"",			-1,				-1,				-1 }		-- Set up results table for passing around	
			local pTargetPlayer = Players[gAiMissionTable[iIndex][7]]						-- Set the target player pointer
			local iMissionNumber = gAiMissionTable[iIndex][6]								-- Set the mission number
			local pTargetCity = gAiMissionTable[iIndex][8]									-- Get the target city pointer
			
			if pTargetCity then																-- If pTargetCity is valid, then					
				local sAttckingAgentName, iSelectedAgentRank = GetAICivSpymaster(pPlayer)	-- Use "Spymaster" since AI may not have a spy in target city at the time; a bit of an advantage...
											
				GetMissionResult(tMissionResultData, iMissionNumber, pPlayer, sAttckingAgentName, sSelectedAgentRank, iCurrentEra, pTargetPlayer, pTargetCity, -2)	-- Get results of AI CO mission; -2 = nil building
								
				ProcessMissionResponses(tMissionResultData, iMissionNumber, pPlayer, pTargetPlayer, pTargetCity, sAttckingAgentName, sSelectedAgentRank)		-- Get the target civ & international response
								
				if (tMissionResultData[5] ~= INFEASIBLE) then								-- Don't charge AI for missions which turned out infeasible (i.e. no Great Persons near city for a Assassinate GP mission)
					local iAiPlayerGold = pPlayer:GetGold()									-- Get total gold AI civ has on hand	
					--print("AI Civ GOH: " .. iAiPlayerGold .. "    mission cost: " .. gAiMissionTable[iIndex][10])					
					iAiPlayerGold = iAiPlayerGold - gAiMissionTable[iIndex][10]				-- Subtract the mission cost from AI player's gold on hand
					if iAiPlayerGold < 0 then iAiPlayerGold = 0 end							-- Prevent negative gold balance					
					pPlayer:SetGold(iAiPlayerGold)											-- Set attacking civ total gold to previous amount less mission cost
					--print("New AI Civ GOH: " .. pPlayer:GetGold() )
				end
				
				if pTargetPlayer:IsHuman() then												-- Only build/display Notification if the human player is the AI CO target
					local sAttackerInfo = "an unidentified civilization"					-- Default attacker ID to report
					local sAppearenceInfo = "appear to be"									-- Default "certainty"
					local iEspionagePosture = CalculateEspionagePosture(pPlayer, pTargetCity) or 0		-- Target civ's EP (0..50 + (# Spies * 2)
					--local iBuildingInfluence = GetBuildingInfluence(pTargetCity) or 0		-- Target City defensive buildings (0..50)
					--local iUnitInfluence = GetUnitInfluence(pTargetPlayer, pTargetCity) or 0				-- # units*1 and Level of Covert Op units * 3

					local iChanceToIDAttacker = iEspionagePosture * 3						-- Chance to ID attacker: changed to straight EP (sman:20180220)
					if iChanceToIDAttacker < 33 then iChanceToIDAttacker = 33 end			-- Added a lower limit chance to ID to 33% (sman:20180222)
					if iChanceToIDAttacker > 75 then iChanceToIDAttacker = 75 end			-- Limit chance to ID to 75% (sman:20180220)

					if math.random(1, 100) < iChanceToIDAttacker then						-- Is the random # less than the chance to ID threshold?  Then,
						sAttackerInfo = pPlayer:GetCivilizationDescription()				-- Set the ID string from unid'd to id'd
						if math.random(1, 100) < iChanceToIDAttacker then sAppearenceInfo = "have been identified, coming" end	-- Check to see if we're certain or guessing
					end
										
					local sAttackerOrigin = "The perpetrators " .. sAppearenceInfo .. " from the " .. sAttackerInfo .. "."		-- Set the OP Fate element to include in the Notification
										
					local sSeparatorString = "[NEWLINE][NEWLINE]"									-- Experiment with how to separate message components
					--local sSeparatorString = "  "
					local sLongMessage = tMissionResultData[3]										-- Build the long message string piece by piece; [3]=Mission Result

					if sAttackerOrigin ~= "" then sLongMessage = sLongMessage .. sSeparatorString .. sAttackerOrigin end				-- Attacker ID string
					if tMissionResultData[4] ~= "" then sLongMessage = sLongMessage .. sSeparatorString .. tMissionResultData[4] end	-- International result string
					
					local iTargetCityX = pTargetCity:GetX() or -1											-- Get X location for target city
					local iTargetCityY = pTargetCity:GetY() or -1											-- Get Y location for target city	

					SendNotification(tMissionResultData[1], sLongMessage, tMissionResultData[2], pUnit, iTargetCityX, iTargetCityY)	-- Issue Notification
				end					
				gAiMissionTable[iIndex][9] = tMissionResultData[3] .. " " .. tMissionResultData[4] or "Status unknown"		-- Use the fade string as the summary message
			else																			-- pTargetCity no longer valid - city doesn't exist or is captured (therefore new pointer)			
				gAiMissionTable[iIndex][9] = "Status unknown"								-- Use the fade string as the summary message; may need to look here further
			end
			gAiMissionTable[iIndex][2] = NO													-- Show the mission as completed ([2]=Active mission? NO)
			--print("Executing a mission. Mission Index: " .. iIndex .. "    Player: " .. pPlayer:GetID() .. "  on game turn: " .. iGameTurn .. "  Planned Turn: " .. gAiMissionTable[iIndex][3] .. "    Target Civ: " .. gAiMissionTable[iIndex][7] .. "    Target City: " .. gAiMissionTable[iIndex][8]:GetName() .. "    Mission Number: " .. iMissionNumber .. "    Results: " .. gAiMissionTable[iIndex][9])

		end	
	end
end


function AIMissionTableGarbageCollection(iGameTurn)											-- Changes status from Open to Closed if the TurnToLaunch is lower (earlier) than current game turn
	for iIndex = 1, #gAiMissionTable do														-- Cycle through all missions in the table
		if (gAiMissionTable[iIndex][2] == YES) and (gAiMissionTable[iIndex][5] < iGameTurn) then	-- Is the Status OPEN and is the TTL earlier than Current Game Turn, then...				
			gAiMissionTable[iIndex][2] = NO													-- Change the status from OPEN to CLOSED
			--print("AI Mission Table Garbage Collection. Turn: " .. iGameTurn .. "  Mission IDX: " .. iIndex .. "    TTL: " .. gAiMissionTable[iIndex][5] .. " - Mission Status changed from OPEN to CLOSED")
		end		
	end
end


function DisplayAIMissionTable(iGameTurn)													-- Display the contents of the AI Mission Table
	print("==============================================================================================================")
	print("AI Mission Table Contents.  Game Turn: " .. iGameTurn)
	if #gAiMissionTable == 0 then
		print("AI Mission Table is currently empty.")
	else		
		local iOpenTally, iClosedTally = 0, 0
		for iIndex = 1, #gAiMissionTable do
			local sMissionStatus = "Open"
			if gAiMissionTable[iIndex][2] == NO then sMissionStatus = "Closed" iClosedTally = iClosedTally + 1
			else
				iOpenTally = iOpenTally + 1
			end
			local sTempCityName = gAiMissionTable[iIndex][8]:GetName()
			if not sTempCityName then sTempCityName = "no longer valid" end
			sString = string.format("Msn #: %5i    Mission: %-15.15s    Attacker: %-12.12s    Status: %-6s    Turn Scheduled: %5i    Launch Turn: %5i    Target: %-15.15s    Target City: %-15.15s    Results: %s",
				iIndex, g_MissionList[gAiMissionTable[iIndex][6]][ML_MISSION_NAME], Players[gAiMissionTable[iIndex][1]]:GetName(), sMissionStatus, 
				gAiMissionTable[iIndex][3], gAiMissionTable[iIndex][5], Players[gAiMissionTable[iIndex][7]]:GetName(), sTempCityName, gAiMissionTable[iIndex][9] )	
			print(sString)			
		end
		print("Total Open Missions in gAiMissionTable Table: " .. iOpenTally)
		print("Total Closed Missions in gAiMissionTable Table: " .. iClosedTally)
	end
	print("==============================================================================================================")	
end


function DisplayCOMissionStats(iGameTurn)													-- Display mission success/fail/prevent stats
	print("==============================================================================================================")
	print("CO Mission Stats.  Game Turn: " .. iGameTurn)	
	local iTotalAttempts, iTotalSuccess, iTotalSuccessRate, iTotalPrevented, iTotalPreventRate = 0, 0, 0, 0, 0
	for iIndex = 2, ABORT_MISSION-1 do		
		if (g_MissionList[iIndex]) then
			local iFails = g_MissionList[iIndex][ML_NUM_ATTEMPTS] - (g_MissionList[iIndex][ML_NUM_SUCCESS] + g_MissionList[iIndex][ML_NUM_PREVENTS])		
			local iSuccessRate = 0 
			if g_MissionList[iIndex][ML_NUM_ATTEMPTS] > 0 then
				iSuccessRate = math.floor( (g_MissionList[iIndex][ML_NUM_SUCCESS] / g_MissionList[iIndex][ML_NUM_ATTEMPTS]) * 100 )
			end

			sString = string.format("Msn: %-25.25s    Attempted: %6i    Successful: %5i (%3i %%)    Failed*: %5i    Prevented: %5i",
					g_MissionList[iIndex][ML_MISSION_NAME], g_MissionList[iIndex][ML_NUM_ATTEMPTS], g_MissionList[iIndex][ML_NUM_SUCCESS], iSuccessRate, iFails, g_MissionList[iIndex][ML_NUM_PREVENTS]  )   
			print(sString)			
		
			iTotalAttempts = iTotalAttempts + g_MissionList[iIndex][ML_NUM_ATTEMPTS]
			iTotalSuccess = iTotalSuccess + g_MissionList[iIndex][ML_NUM_SUCCESS]
			iTotalPrevented = iTotalPrevented + g_MissionList[iIndex][ML_NUM_PREVENTS]
		end
	end	
	print("-----")
	if iTotalAttempts > 0 then
		iTotalSuccessRate = math.floor ( ( iTotalSuccess / iTotalAttempts ) * 100	)
		iTotalPreventRate = math.floor ( ( iTotalPrevented / iTotalAttempts ) * 100	)
	end
	print("A Mission Table Size: " .. #gAiMissionTable .. "   Total Attempts: " .. iTotalAttempts .. "    Total Success: " .. iTotalSuccess .. "  ( " .. iTotalSuccessRate .. "% )    Total Prevented: " .. iTotalPrevented .. "  ( " .. iTotalPreventRate .. "% )" )
	print("==============================================================================================================")	
end

function DisplayIncidentStats(iGameTurn)	
	print("==============================================================================================================")
	print("Incident Stats.  Game Turn: " .. iGameTurn)	
	local iTally = 0
	for iIndex = 1, #g_IncidentStats do				
		sString = string.format("Incident: %-22.22s    Count: %6i", g_sIncidentName[iIndex], g_IncidentStats[iIndex] )
		print(sString)	
		iTally = iTally + g_IncidentStats[iIndex]
	end	
	print("A Mission Table Size: " .. #gAiMissionTable .. "    Total Status in g_IncidentStats Table: " .. iTally .. "    Non-NoResponses: " .. iTally - g_IncidentStats[9])
	print("==============================================================================================================")
end

-- =================================================================================
--			Process AI Covert Missions
--
--  Add function that drives AI CO missions - AI can use all missions except #1 (Conduct Recon)
-- 
-- =================================================================================

function PlayerDoTurn_ProcessAI_CovertOperations( iPlayer )									-- Only major AI civs can use covert missions.  Change to let city states launch?
	local pPlayer = Players[iPlayer]														-- Get pPlayer from player ID				
	local iGameTurn = Game.GetGameTurn()													-- Get current game turn
	local iCurrentEra = pPlayer:GetCurrentEra()												-- Get current player's Era level
		
	-- =========================================================================================================================
	-- Temp routines for testing

	--if (iPlayer == 0) then																	-- Perform various bookkeeping/testin functions during the human player turn
	--if (pPlayer:IsHuman()) and (iGameTurn % 5 == 0) then										-- Display the AI mission table/mission stats tables - used during normal human playing
		--	print("Game Turn: " .. iGameTurn)	
		--if (iGameTurn % 50 == 0) then														-- Display the AI mission table/mission stats tables - used during LiveTuner		
			--DisplayCOMissionStats(iGameTurn)												-- Display Mission stats
			--DisplayIncidentStats(iGameTurn)
			--DisplayAIMissionTable(iGameTurn)												-- Display AI Mission Table - printing this often leads to CTDs for some reason...			
		--end
		--if (iGameTurn % 50 == 0) then														
			--AIMissionTableGarbageCollection(iGameTurn)										-- Close out open missions if the current game turn is larger than the TurnToLaunch
		--end
		--return																				-- Prevent player=0 causing issues during LiveTuner test games
	--end						
	
	-- =========================================================================================================================
	-- Process AI Covert Operation Mission Components
	
	if (pPlayer:IsAlive()) and (not pPlayer:IsHuman()) and (not pPlayer:IsMinorCiv()) and (not pPlayer:IsBarbarian()) then	-- Process AI covert operations only for living major AI civs	
	
		Component1_Schedule_Mission(pPlayer, iGameTurn, iCurrentEra)						-- AI Process Component 1: Schedule an AI CO Mission:  To launch or not to launch, that is the question...		
		Component3_Execute_Mission(pPlayer, iGameTurn, iCurrentEra)							-- AI Process Component 3: Process an AI CO Mission
	
	elseif pPlayer:IsHuman() then
		
		Component2_Warn_Mission(pPlayer, iGameTurn, iCurrentEra)							-- AI Process Component 2: Warn human player only of an Upcoming AI CO Mission		

	end

end
GameEvents.PlayerDoTurn.Add( PlayerDoTurn_ProcessAI_CovertOperations )
