-- Process_International_Response.lua
-- Author: sman
-- DateCreated: 01/08/18 9:19:38 AM
--------------------------------------------------------------

-- =================================================================================================================================
-- Reponse global data

-- Incidence Severity Enumerations:  Minor (1/2= 25%)  Denounce (3/4 = 35%)  War (5/6/7 = 35%)  Global War (8 = 5%)
RESPONSE_MUTED = 1																			-- 10%														
RESPONSE_MINOR = 2																			-- 10%
RESPONSE_DENOUNCE = 3																		-- 30%
RESPONSE_DENOUNCE_WITH_ALLIES = 4															-- 10%
RESPONSE_WAR = 5																			-- 15%
RESPONSE_WAR_WITH_ALLIES = 6																--  5%
RESPONSE_WAR_PERMANENT = 7																	-- 15%
RESPONSE_WAR_GLOBAL = 8																		--  5%

--g_tIncidenceResponse = {  [RESPONSE_MUTED] = 1, [RESPONSE_MINOR] = 10, [RESPONSE_DENOUNCE] = 20, [RESPONSE_DENOUNCE_WITH_ALLIES] = 50, [RESPONSE_WAR] = 60, 
--							[RESPONSE_WAR_WITH_ALLIES] = 75, [RESPONSE_WAR_PERMANENT] = 80, [RESPONSE_WAR_GLOBAL] = 95	}


g_tIncidenceResponse = {	[RESPONSE_MUTED] = 1, [RESPONSE_MINOR] = 20, [RESPONSE_DENOUNCE] = 40, [RESPONSE_DENOUNCE_WITH_ALLIES] = 72, [RESPONSE_WAR] = 82, 
							[RESPONSE_WAR_WITH_ALLIES] = 92, [RESPONSE_WAR_PERMANENT] = 94, [RESPONSE_WAR_GLOBAL] = 97	}


g_iDomesticResponseProbability = 33															-- Percent chance of domestic resistance to Covert Ops; adjusted later by excess happiness
g_iAlliedReponseProbability = 33															-- Percent chance allies will denounce or declare war as a response to an incident
g_iGlobalWarResponseProbability = 25														-- Percent chance any civ will declare war as a result of an incident when RESPONSE_WAR_GLOBAL is selected
g_iMinimumChanceOfIncident = 1																-- Percent chance floor of any response for any mission

--TEMP
--g_iAlliedReponseProbability = 88															-- Percent chance allies will denounce or declare war as a response to an incident
--g_iGlobalWarResponseProbability = 77														-- Percent chance any civ will declare war as a result of an incident when RESPONSE_WAR_GLOBAL is selected


-- =================================================================================================================================
-- Functions

function GetIncidentOutcome(iMissionNumber, pAttackingPlayer, pTargetPlayer, pTargetCity)	-- Determines what happened during the covert mission (successful? operator survive? incident?)
	local iIncidentResult = NO																-- Default result is there was no incident	
	-- Return "ATWAR" response if target player and active player already at war; otherwise test for incident based on formula
	local pTargetTeam = Teams[pTargetPlayer:GetTeam()]										-- Get team instance for target civ
	local iAttackingTeamID = pAttackingPlayer:GetTeam()										-- Get Team ID for the attacking civ	
	local iMissionLevel = g_MissionList[iMissionNumber][ML_MISSION_LEVEL] or 1				-- Level of the attack

	if  pTargetTeam:IsAtWar(iAttackingTeamID) then											-- Are teams at war?  Then...
		iIncidentResult = ATWAR																-- Change return value to ATWAR
	else																					-- If not at war, then...		
		local iAttackerEspionagePosture = CalculateEspionagePosture(pAttackingPlayer, pTargetCity)	-- Get the EP for the attacker in target city
		local iDefenderEspionagePosture = CalculateEspionagePosture(pTargetPlayer, pTargetCity)	-- Get the EP for the defender in target city
		local iAdjustedIncidentChance = g_MissionList[iMissionNumber][ML_CHANCE_INCIDENT] + iDefenderEspionagePosture - iAttackerEspionagePosture	-- Basic chance of incident - Defense IP + Attack IP
		
		if iAdjustedIncidentChance < g_iMinimumChanceOfIncident then iAdjustedIncidentChance = g_iMinimumChanceOfIncident end
		local iIncidentChance = math.random(1, 100)
		if iIncidentChance < iAdjustedIncidentChance then									-- Did mission cause Incident?  Based on chance for specific mission
			iIncidentResult = YES															-- Change default return value 
		end
		--print("====> Incident Results: " .. iIncidentResult .. "    Basic chance of incident: " .. g_MissionList[iMissionNumber][ML_CHANCE_INCIDENT] .. "    Attack EP: " .. iAttackerEspionagePosture .. "    Defense EP: " .. iDefenderEspionagePosture .. "    IadjIncChance: " .. iAdjustedIncidentChance .. "    Incident Chance: " .. iIncidentChance)
	end	
	return iIncidentResult
end


function GetResponseSeverity(iMissionNumber, iMissionResult, pTargetPlayer, iIncidentResult)	-- Return an index for the g_tIncidenceResponse table
	local iSeverityIDX = 0																	-- Return value, default to 0
	local iMissionLevel = g_MissionList[iMissionNumber][ML_MISSION_LEVEL] or 1				-- Level of the attack
	local iResponseSeverity = math.random(1, 100)											-- Just how severe is the aftermath?								
	--print ("Unadjusted Incident Severity: " .. iResponseSeverity)

	-- Adjust to the severity based on mission level.  The higher level the mission, the higher the chance for a more serious response - these stack on top of normal severity number!
	-- Unsuccessful range: L1=1..5, L2=2..10, L3=3..15, L4=4..20, L5=5..25;   successful range: L1=5..10, L2=10..20, L3=15..25, L4=20..30, L5=25..50
	local iLowerBound = 0																	-- Unsuccessful mission lower bound of random impact range
	local iUpperBound  = 3																	-- Unsuccessful mission upped bound of random impact range										
	if iMissionResult == YES then 															-- Increase random impact range for SUCCESSFUL missions		
		iLowerBound = 2																	
		iUpperBound = 5 			
	end		
	local iSeverityAdjustment = math.random(iMissionLevel * iLowerBound, iMissionLevel * iUpperBound)	-- CO ML_MISSION_LEVEL (1..5)
	iResponseSeverity = iResponseSeverity + iSeverityAdjustment
	--print ("Adjusted Incident Severity: " .. iResponseSeverity .. "    Mission Level: " .. iMissionLevel .. "    Incident Result: " .. iIncidentResult)		
	
	if iIncidentResult == ATWAR then														-- Parties were already at war, so no need to Denounce/DoW again...
		iResponseSeverity = 15																-- Safe number, in the middle of the RESPONSE_MINOR range
	end
					
	local iSeverityIDX = #g_tIncidenceResponse												-- Set the index to the last element in the "incidence response" table	
	for idx = 1, #g_tIncidenceResponse - 1 do												-- Move through the incidence response table from frist element until next-to-last
		if (iResponseSeverity >= g_tIncidenceResponse[idx]) and (iResponseSeverity < g_tIncidenceResponse[idx+1]) then		-- If the score is between this element and the next, then
			iSeverityIDX = idx																-- Set the return index to the loop index value
			break																			-- Terminate the loop early, we've found what we're looking for
		end
	end			
	--print("Incident Severity.  Working Severity: " .. iResponseSeverity .. "    Severity: " .. g_sIncidentName[iSeverityIDX] .. "   Upperbound: " .. iUpperBound .. "   Severity Adjust, based on mission level: " .. iSeverityAdjustment)				
	
	if iSeverityIDX == RESPONSE_WAR_GLOBAL then												-- Adjust most Global Wars to War, War with Allies, perma War
		local iTempAdjust = math.random(1, 100)												-- The chance for GW is supposed to be only 3%, 		
		if iTempAdjust< 30 then																-- but since it's the last enumeration, any wild overflows of 
			iSeverityIDX = RESPONSE_DENOUNCE												-- severity chooses it by default.
		elseif iTempAdjust< 60 then															
			iSeverityIDX = RESPONSE_WAR																
		elseif iTempAdjust< 75 then															-- This block seeks to ensure GW isn't happening tooooo often
			iSeverityIDX = RESPONSE_WAR_WITH_ALLIES											-- Only 5% chance Global War stays Global
		elseif iTempAdjust< 95 then															
			iSeverityIDX = RESPONSE_WAR_PERMANENT 
		end		
	end

	if pTargetPlayer:IsMinorCiv() then														-- Adjust Severity for a City State target	
		local iOldSeverityIDX = iSeverityIDX
		if iSeverityIDX == RESPONSE_DENOUNCE then iSeverityIDX = RESPONSE_MINOR end
		if iSeverityIDX == RESPONSE_DENOUNCE_WITH_ALLIES then iSeverityIDX = RESPONSE_WAR end
		if iSeverityIDX == RESPONSE_WAR_PERMANENT then iSeverityIDX = RESPONSE_WAR end
		if iSeverityIDX == RESPONSE_WAR_GLOBAL then iSeverityIDX = RESPONSE_WAR end
		--print("City State target.  iSeverityIDX changes from " .. iOldSeverityIDX .. " to " .. iSeverityIDX)	
	end
	
	--print("Incident Severity.  Final Severity: " .. iResponseSeverity .. "    Severity: " .. g_sIncidentName[iSeverityIDX] )				
	g_IncidentStats[iSeverityIDX] = g_IncidentStats[iSeverityIDX] + 1
	return iSeverityIDX
end

-- ===================================================================================================
--		Reponses

function ProcessLimitedResponse(pAttackingPlayer)											-- Generate a randomized "non-response"
	local sAdjectives = { "muted", "minor", "muzzled", "subdued", "moderate", "faint", "mild", "imperceptible", "muffled", "slight", "pro-forma" }
	local sNouns = { "responses to", "reactions concerning", "acknowledgment of", "retorts to", "reverberations from", "ripostes to", "murmurs about", "murmurings concerning", "yawns in regards to", "reprisals from" }
	local sEvent = { "incident", "affair", "event", "operation", "attack", "adventure", "episode", "matter", "strike", "offense", "undertaking" }
	local sEvent2 = { "implications from", "ramifications from", "connections with", "entanglements from", "reprecussions from", "accusations of blame associated with", "accusations about" }
	local sVerb = { "escaped", "avoided", "deflected", "side stepped", "parried", "evaded", "circumvented", "fended off", "dodged" }
	local sActor = { "operative", "spy", "mole", "agent", "agitators", "seditionists", "spy", "agent", "operative", "operatives", "spies", "agents" }

	local sReturnString = ""
	if pAttackingPlayer:IsHuman() then														-- Added better defualt return message, if attacker is human (sman:20180211)
		sReturnString = "Your " .. sActor[math.random(1, #sActor)] .. " " .. sVerb[math.random(1, #sVerb)] .. " all " .. sEvent2[math.random(1, #sEvent2)] .. " this " .. sEvent[math.random(1, #sEvent)] .. "."
	else		
		sReturnString = "There have been only " .. sAdjectives[math.random(1, #sAdjectives)] .. " " .. sNouns[math.random(1, #sNouns)] .. " this " .. sEvent[math.random(1, #sEvent)] .. "."
	end
	return sReturnString
end


function GetRespondingCivTable(iMissionNumber, pAttackingPlayer, pTargetPlayer, iSeverityIDX)	-- Covers DoWs for friends and Global War
	local pAttackingPlayerTeam = Teams[pAttackingPlayer:GetTeam()]							-- Get team pointer for atacking player
	local tRespondingCivs = {}																-- Table holding playerIDs of those responding
	local iTargetPlayerID = pTargetPlayer:GetID()											-- Get the target civ's team ID
	local iTargetCivResponseProbability = (g_MissionList[iMissionNumber][ML_MISSION_LEVEL] * 15) + 15	-- Mission levels: 1=30%, 2=45%, 3=60%, 4=75%, 5=90%
	--print("GRT. Severity: " .. g_sIncidentName[iSeverityIDX] .. "   Attacker: " .. pAttackingPlayer:GetCivilizationShortDescription() .. "    Defender: " .. pTargetPlayer:GetCivilizationShortDescription())
			
	for idx = 1, GameDefines.MAX_CIV_PLAYERS-1, 1 do										-- Loop through all AI players (skip human = 0, barbs trapped later)				
		local bDoResponse = false															-- Default to false		
		local iUpperBound = 100																-- Upper limit of random range for major civs
		if Players[idx]:IsMinorCiv() then iUpperBound = 200 end								-- Upper limit of random range for minor civs
		local pTempPlayer = Players[idx]													-- Get the loop players instance
		local pTempTeam = Teams[pTempPlayer:GetTeam()]										-- Get team pointer for loop player
		if ( pTempPlayer:IsAlive() ) and													-- Only test for living civs
				( pTempTeam:IsHasMet(pAttackingPlayer:GetTeam()) ) and						-- Has loop civ met Attcker?
				( pTempTeam:IsHasMet(pTargetPlayer:GetTeam()) ) and							-- Has loop civ met Target civ?
				( not pTempTeam:IsBarbarian() ) and											-- Civ not barb?
				( pAttackingPlayer:GetID() ~= idx ) then									-- Is the loop civ not the attacker?						
								
			if iTargetPlayerID == idx then													-- If loop civ is Target civ, then...;  Always include target civ in any war tests
					bDoResponse = true		-- automatically true for target civ; othewise this function would not be called.

			elseif ( (iSeverityIDX == RESPONSE_DENOUNCE_WITH_ALLIES) or						-- If event type is with allies, test if loop civ is DoF with target civ
					(iSeverityIDX == RESPONSE_WAR_WITH_ALLIES) )  and
					(iTargetPlayerID ~= idx) then											-- Should have screened target player in initial IF
					 									
				if pTempPlayer:IsDoF(iTargetPlayerID) then									-- Loop Civ has declared DOF with target civ?
					if math.random(1, iUpperBound) < g_iAlliedReponseProbability then		-- Random # below threshold, civ will respond!
						bDoResponse = true
					end			
				end
			elseif ( iSeverityIDX == RESPONSE_WAR_GLOBAL ) and 
				( (iTargetPlayerID ~= idx) )	then									-- If Global Event, and loop civ has denounced, higher chance of responding
				if pTempPlayer:IsDenouncingPlayer(pAttackingPlayer:GetID()) then			-- Has loop player denounced Attacking player?  if so, then use delta between allies and all others' probabilities - split the difference
					if math.random(1, iUpperBound) < (g_iAlliedReponseProbability - g_iGlobalWarResponseProbability) then		-- Random # below threshold, civ will respond!
						bDoResponse = true
					end
				else																		-- Otherwise, a non-Denouncing civ...
					if math.random(1, iUpperBound) < g_iGlobalWarResponseProbability then	-- Random # below threshold, civ will respond!
						bDoResponse = true
					end
				end
			end
			if bDoResponse then																	-- If any of the tests worked, add to response table						
				--print("Adding civ " .. pTempPlayer:GetCivilizationShortDescription() .. " to the response table")
				tRespondingCivs[#tRespondingCivs + 1] = idx
			end
		end
	end	
	return tRespondingCivs
end


function ProcessSeriousResponses(tRespondingCivs, iSeverityIDX, pAttackingPlayer, pTargetPlayer)
	local iAttackingPlayerID
	local pAttackingPlayerTeam = Teams[pAttackingPlayer:GetTeam()]											-- Get team pointer for attacking player
	local iAttackingPlayerTeamID = pAttackingPlayerTeam:GetID()											-- Get team pointer for attacking player
	local sEventDescription = " is now at war with "
	if (iSeverityIDX == RESPONSE_DENOUNCE) or (iSeverityIDX == RESPONSE_DENOUNCE_WITH_ALLIES ) then
		sEventDescription = " has been denounced by "
	end

	local sReturnString = pAttackingPlayer:GetCivilizationShortDescription() .. sEventDescription	-- Start string with Attacker + EventDescription
	for idx = 1, #tRespondingCivs, 1 do														-- Loop through all AI players in the table
		local iRespondingPlayerID = tRespondingCivs[idx]									-- Get ID for loop player
		local pRespondingPlayer = Players[iRespondingPlayerID]								-- Set pointer for loop player
		local sDuration = ""																-- Set the text description of type of war (normal or perma)

		if (iSeverityIDX == RESPONSE_DENOUNCE) or											-- If the response is a Denounce of Allied Denounce, then...
				(iSeverityIDX == RESPONSE_DENOUNCE_WITH_ALLIES ) then
			--print("Player: " .. pRespondingPlayer:GetCivilizationShortDescription() .. " Denouncing...")
			pRespondingPlayer:DoForceDenounce(iAttackingPlayerID)							-- Force denouncement by this player against the attacker
			
		elseif (iSeverityIDX == RESPONSE_WAR ) or											-- If the response if war of any kind, then...
				(iSeverityIDX == RESPONSE_WAR_WITH_ALLIES ) or 
				(iSeverityIDX == RESPONSE_WAR_PERMANENT ) or 
				(iSeverityIDX == RESPONSE_WAR_GLOBAL) then				
			--print("Player: " .. pRespondingPlayer:GetCivilizationShortDescription() .. " DoWing...")
			pAttackingPlayerTeam:DeclareWar(pRespondingPlayer:GetTeam())					-- Ensure CO Attacker gets Warmonger penalty
			
			if (iSeverityIDX == RESPONSE_WAR_PERMANENT ) and ( pTargetPlayer:GetID() == tRespondingCivs[idx] ) then	-- Response type is Permanent War; only target civ can perma war
				pAttackingPlayerTeam:SetPermanentWarPeace(pRespondingPlayer:GetTeam(), true)	-- Set war status to permanent
				sDuration = " (PERMANENT)"													-- Adjust the text description
			end
		end
		
		local sSeperator = ", "																-- Set up seperators, based on which entry in the list this is	
		if (idx == 1) then																	-- There is exactly 1 civ in table 
			sSeperator = ""
		elseif (#tRespondingCivs == 2) and (idx == 2) then									-- There are exactly 2 civs in table & this is the first one
			sSeperator = " and "															-- So prepend string with " and "		
		elseif (#tRespondingCivs > 2) and (idx == #tRespondingCivs) then					-- There are 2 or more civs in table, and this is the last one
			sSeperator = ", and "  										-- So prepend string with "and"
		end

		sReturnString = sReturnString .. sSeperator .. pRespondingPlayer:GetCivilizationShortDescription() .. sDuration

		--print("Working string.  civ: " .. pRespondingPlayer:GetCivilizationShortDescription() .. "   string: " .. sReturnString)

	end

	if #tRespondingCivs == 0 then															-- Adjust the return message, based on how many civs responded
		sReturnString = ProcessLimitedResponse(pAttackingPlayer)							-- If no civs responded, genericize the message
	end

	sReturnString = sReturnString .. "."
	--print("Return String: " .. sReturnString)
	return sReturnString
end


function ProcessDomesticTurmoil(pAttackingPlayer, iSeverityIDX, iIncidentResult)			-- Process domestic response, if any, to a mission
	local sReturnString = ""
	local iDomesticTurmoilTurns = iSeverityIDX - 1											-- Potential "domestic turmoil" turns based on severity of international response (only happens for Denounce or higher, length 1..6 turns)
	if iDomesticTurmoilTurns < 1 then iDomesticTurmoilTurns = 0 end							-- If less than 1, there is no domestic response

	if iDomesticTurmoilTurns > 0 then														-- If we have a domestic response, then...		
		local iResponseFloor = 15															-- Minimum chance of a domestic response
		if iIncidentResult == ATWAR then iResponseFloor = 5 end								-- Lower minimum chance if at already war with target

		local iResponseChance = g_iDomesticResponseProbability - pAttackingPlayer:GetExcessHappiness()	-- Lower the chance of response by excess happiness; g_iDomesticChance=33
		if iResponseChance < iResponseFloor then iResponseChance = iResponseFloor end		-- Ensure at least xx% (iResponseFloor) chance of response
		
		if math.random(1, 100) < iResponseChance then										-- Check random result against chance of response				
			local pAttackingCapitalCity = pAttackingPlayer:GetCapitalCity()					-- Get the capital city instance
			local iResistanceDuration = math.random(1, iDomesticTurmoilTurns)				-- Set duration of Resistance for 1..iDomesticTurmoilTurns  (should be 6 max)
			pAttackingCapitalCity:ChangeResistanceTurns(iResistanceDuration)				-- These resistance turns are cumulative!  This action adds turns to previous Resistance
			sReturnString = "[NEWLINE][NEWLINE]The citizens of the " .. pAttackingPlayer:GetCivilizationAdjective() .. " capital are protesting their leadership (" .. iResistanceDuration .. " turns)."
		end	
	end	
	return sReturnString
end

--[[
function OnCOGetAttackDefendModifier(ePlayer1, ePlayer2) --swapped the arguments as this will inflict a modifier from the defender to the attacker (assuming that's what you want ofc)
	--print("Inside OGADM!  GT: " .. Game.GetGameTurn() .. "    ePlayer1: " .. ePlayer1 .. "    ePlayer2: " .. ePlayer2 .. "   Return val: " .. g_Diplomacy_Adjustment[ePlayer1][ePlayer2])
	return g_Diplomacy_Adjustment[ePlayer1][ePlayer2]
end
GameEvents.GetScenarioDiploModifier1.Add(OnCOGetAttackDefendModifier)


function AdjustDiplomaticRelations(pAttackingPlayer, pTargetPlayer, iMissionNumber, iSeverityIDX)
	if iSeverityIDX == RESPONSE_MUTED then return end										-- No diplo adjustments for response #1
	local iMissionLevel = g_MissionList[iMissionNumber][ML_MISSION_LEVEL] or 3				-- Get mission level used against target
	local iAttackerID = pAttackingPlayer:GetID()
	local iAdjustment = math.floor(( iMissionLevel * -20 )	/ (10 - iSeverityIDX ))			-- Based on Level/Severity: L1/Severity 1 = -2,  L5/Severity 8 =-50
	
	if pTargetPlayer:IsMinorCiv() then														-- Adjust diplo modifier for a City State target		
		local iOldRelationship = pTargetPlayer:GetMinorCivFriendshipLevelWithMajor(iAttackerID)
		pTargetPlayer:ChangeMinorCivFriendshipWithMajor(iAttackerID, iAdjustment)
		local iNewRelationsip = pTargetPlayer:GetMinorCivFriendshipLevelWithMajor(iAttackerID)
		--print("Adjusting City State relationship betweeen " .. pTargetPlayer:GetCivilizationShortDescription() .. " and " .. pAttackingPlayer:GetCivilizationShortDescription() .. "   Was: " .. iOldRelationship .. "   Now is: " .. iNewRelationsip .. "    Adjustment: " .. iAdjustment)
	else																					-- Adjust diplo modifier for a Major Civ target
		g_Diplomacy_Adjustment[pAttackingPlayer:GetID()][pTargetPlayer:GetID()] = g_Diplomacy_Adjustment[pAttackingPlayer:GetID()][pTargetPlayer:GetID()] + (iAdjustment * -1)	-- Set modifier with positive value, as this means "bad" in the game's diplo system
		g_Diplomacy_Adjustment[pTargetPlayer:GetID()][pAttackingPlayer:GetID()] = g_Diplomacy_Adjustment[pTargetPlayer:GetID()][pAttackingPlayer:GetID()] + (iAdjustment * -1)
		--print("Adjusting betweeen " .. pTargetPlayer:GetCivilizationShortDescription() .. " and " .. pAttackingPlayer:GetCivilizationShortDescription() .. "   Adjustment: " .. iAdjustment * -1)
	end
end
]]--

-- =============================================================================================================================================
--		Main Function

function ProcessMissionResponses(tMissionResultData, iMissionNumber, pAttackingPlayer, pTargetPlayer, pTargetCity, sSpyName, iSpyRank)	
	local iIncidentResult = NO																-- Default response is NO
	local sResultMessage = ""																-- Clear the response message

	--print("=====================================================================================================================")  
	--print("===============  Processing new mission: " .. g_MissionList[iMissionNumber][ML_MISSION_NAME] .. "   Atk: " .. pAttackingPlayer:GetCivilizationShortDescription() .. "    Def: " .. pTargetPlayer:GetCivilizationShortDescription())
		
	iIncidentResult = GetIncidentOutcome(iMissionNumber, pAttackingPlayer, pTargetPlayer, pTargetCity)	-- Determine response YES/NO

	if iIncidentResult ~= NO then															-- If YES or ATWAR, then...  A response of any kind assumes target civ ID'ed attackers 
		local iSeverityIDX = GetResponseSeverity(iMissionNumber, iMissionResult, pTargetPlayer, iIncidentResult )	-- How severe was the response (Index of g_tIncidenceResponse)
		
		if (iSeverityIDX == RESPONSE_MUTED) or (iSeverityIDX == RESPONSE_MINOR) then		-- If minor response, then...
			sResultMessage = ProcessLimitedResponse(pAttackingPlayer)						-- Get randomized mumbo-jumbo message
		else																				-- If response is not minor, then...
			tRespondingCivs = GetRespondingCivTable(iMissionNumber, pAttackingPlayer, pTargetPlayer, iSeverityIDX)	-- Build table of responding civ IDs
			if #tRespondingCivs > 0 then													-- If more than NONE civs responded, then...
				sResultMessage = ProcessSeriousResponses(tRespondingCivs, iSeverityIDX, pAttackingPlayer, pTargetPlayer)	-- Implement the results (Denounce/DoW)
			else					
				g_IncidentStats[10] = g_IncidentStats[10] + 1								-- Increment statistics table; [10]=response changed
				sResultMessage = ProcessLimitedResponse(pAttackingPlayer)					-- No civs responded!
			end
		end
				
		local sDomesticResponse = ProcessDomesticTurmoil(pAttackingPlayer, iSeverityIDX, iIncidentResult)	-- Check for domestic response
		sResultMessage = sResultMessage .. sDomesticResponse

		--AdjustDiplomaticRelations(pAttackingPlayer, pTargetPlayer, iMissionNumber, iSeverityIDX)	-- Removed 20190406 (sman) - just doesn't work...

	else																					-- Result == NO
		sResultMessage = ProcessLimitedResponse(pAttackingPlayer)							-- If there was no response to the mission, add a generic response
		g_IncidentStats[9] = g_IncidentStats[9] + 1											-- Increment statistics table; [9]="No Response"
	end
	
	tMissionResultData[4] = sResultMessage													-- Send back the message in mission result table
	tMissionResultData[6] = iIncidentResult													-- Send back the response result in mission result table

end
