-- 40 - Process INTREPs
-- Author: sman
-- DateCreated: 02/19/18 5:11:37 PM
--------------------------------------------------------------


function BuildWarningString(pPlayer, iGameTurn, iIndex, iEspionagePosture, bIsRUMINT)		-- Builds a warning message for Notifications; adds ambiguity to a report based on pPlayers Espionage Posture
																							-- Last argument="Is this a warning" if true, then future/impending report against human player (RUMINT); if false then past reporting (FIR)
	local sWarnString = ""																	-- Return warning message value	
	local sFadeString = ""																	-- Return fade value
	
	local sRumorReport = "Operation results unknown."										-- What is the default report value?
	local sRumorReporter = "anonymous"														-- Who is issuing the report?
	local sRumorAttacker = "unidentified"													-- Who is attacking in the report?
	local sRumorLocation = "a distant land"													-- Where did they attack?  City?  Civ?				
	local sRumorTurn = "recently"															-- Game turn mission executed	
	if bIsRUMINT then sRumorTurn = "soon" end												-- Change Generig GT description from past to future for RUMINT
	local iReportQuality = 0																-- A tally of the number of specific items were included in a report
	
	local sActualReporter																	-- Will contain the adjective of reporting civ, either target (if known) or attacker (if known)
	if Teams[pPlayer:GetTeam()]:IsHasMet(gAiMissionTable[iIndex][7]) then					-- Has pPlayer met the target civ [7]?  If so, then they are the reporter		
		sActualReporter = Players[gAiMissionTable[iIndex][7]]:GetCivilizationAdjective()	-- Set reporting civ to the adjective of actual target civ;  [7] = iTargetPlayerID
	elseif Teams[pPlayer:GetTeam()]:IsHasMet(gAiMissionTable[iIndex][1]) then				-- ElseIf: Has pPlayer met the attacking civ [1]?  If so, then they are the reporter		
		sActualReporter = Players[gAiMissionTable[iIndex][1]]:GetCivilizationAdjective()	-- Adjective of actual target civ;  [7] = iTargetPlayerID	
	else																					-- If pPlayer (human) hasn't met either civ, abandon the report
		return "Failed Report - Unknown Civs", "Failed Report - Unknown Civs", -1, -1, -1	-- Ensure values depict abandoned report; return early
	end
	
	local sActualTargetCiv = Players[gAiMissionTable[iIndex][7]]:GetCivilizationShortDescription() or "unknown civ"	-- Name of actual target civ;  [7] = iTargetPlayerID
	local sActualTargetCity = gAiMissionTable[iIndex][8]:GetName() or "unknown city"		-- Name of actual target city;  [8] = target city pointer
	local sActualAttacker = Players[gAiMissionTable[iIndex][1]]:GetCivilizationAdjective() or "unknown attacker"	-- Adjective of actual target civ;  [1] = mission owner PlayerID
	
	local pCapCity = pPlayer:GetCapitalCity()												-- Get the capital city for pPlayer 	
	local iX = pCapCity:GetX()																-- Default the notification X to the pPlayer capital city X
	local iY = pCapCity:GetY()																-- Default the notification Y to the capital city Y	
					
	local iReportStrenth = iEspionagePosture + (math.random(1, 50) - 25)					-- Strength of Report: based on pPlayer Espionage Posture, with random adjustment of -25..+25; very wide swing - intentional  (changed 20180215:sman)
	if iReportStrenth < 10 then iReportStrenth = 10	end										-- Minimum chance of getting good report is 10%
	
	if bIsRUMINT then																		-- For "RUMINT" (attacks against the human player) the upper confidence bound is higher
		if iReportStrenth > 85 then iReportStrenth = 85 end									-- Maximum chance of getting good report is 80% for true warnings; changed to 85% (sman:20180209)
	else																					-- For "FIR" (attacks happening elsewhere in the world) the confidence limit is lower
		if iReportStrenth > 66 then iReportStrenth = 66 end									-- Maximum chance of getting good report is 50% for normal reports; changed to 66% (sman:20190209)
	end

	-- Determine the specificity of the report, based on the iReportStrength	
	if math.random(1, 100) < iReportStrenth then											-- Chance of knowing WHO reported; defaults to "unk"
		sRumorReporter = sActualReporter													-- Change WHO reported to the actual target civ
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of knowing WHO attacked; defaults to "unk"
		sRumorAttacker = sActualAttacker													-- Change WHO attacked to the actual attacking civ
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of knowing WHERE attacked; defaults to "distand land"
		if math.random(1, 100) < iReportStrenth  then										-- Need "location" - determine if reporting the City or Civilization
			sRumorLocation = sActualTargetCity												-- Return the target city
			if (gAiMissionTable[iIndex][8]) then											-- If the target city exists (i.e. not captured, razed, etc.), then; only change Notification location if we actually know the city
				iX = gAiMissionTable[iIndex][8]:GetX()										-- Set the notification X to the target city X; [8] = pTargetCity
				iY = gAiMissionTable[iIndex][8]:GetY()										-- Set the notification Y to the target city Y
			end
		else
			sRumorLocation = sActualTargetCiv												-- Return the civ as the attack location, not the city
		end
		if not sRumorLocation then sRumorLocation = sActualTargetCiv end					-- If for some reason the location is nil, return the civ		
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end
	
	if math.random(1, 100) < iReportStrenth then											-- Chance of knowing mission results; defaults to "Results unk"
		sRumorReport = gAiMissionTable[iIndex][9]											-- Change generic unk result to the actual results
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of knowing actual mission game turn
		local iAdjustedTurn = gAiMissionTable[iIndex][5] + math.random(1, 3) - 2			-- Adjust the TurnToExecute [5] game turn by +- 1
				
		if bIsRUMINT then																	-- For RUMINT, not too far in the past
			--if iAdjustedTurn > Game.GetGameTurn() then iAdjustedTurn = Game.GetGameTurn() end	-- Ensure no "future reporting"
		else																				-- For FIR, not in the future
			if iAdjustedTurn > Game.GetGameTurn() then iAdjustedTurn = Game.GetGameTurn()-1 end	-- Ensure no "future reporting"		
		end
				
		sRumorTurn = "on game turn " .. iAdjustedTurn										-- Set the value to the adjusted GT value		
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end
	
	if bIsRUMINT then																		-- RUMINT: Adjust the warning ambiguity to more details that a home-civ should have	
		local sMissionType = "unknown"														-- Default mission type to report
		local iMissionChances = math.random(1, 100)											-- Chance of discovering exactly what the enemy is up to
		if iMissionChances < math.floor(iReportStrenth / 2) then							-- The iMissionsChance is less than half of the successful report number
			sMissionType = "'" .. g_MissionList[gAiMissionTable[iIndex][6]][ML_MISSION_NAME] .. "'"		-- Get the exact name of impending attack
		elseif 	iMissionChances < iReportStrenth then
			sMissionType = "Level " .. g_MissionList[gAiMissionTable[iIndex][6]][ML_MISSION_LEVEL]		-- Returns the level of the planned attack
		end
		
		if sRumorLocation == "a distant land" then sRumorLocation = sActualTargetCiv end	-- Ensure the target location is either a city or civ; no "unknowns" allowed
		--if sRumorTurn == "recently"	then sRumorTurn = "soon" end							-- If we don't have specific game turn, change default to a future time
		
		sWarnString = "RUMINT: Your Counter-Intelligence services report " .. sRumorAttacker .. " spies are planning to conduct a " .. sMissionType .. " Covert Operation against " .. sRumorLocation .. " " .. sRumorTurn .. "."
		sFadeString = "RUMINT: Possible Covert Op against " .. sRumorLocation
	
	else																					-- FIR: If not a "true" warning, insert "ambiguity" into the report
		sWarnString ="FIR: Reports by " .. sRumorReporter .. " sources state that " .. sRumorTurn .. ", " .. sRumorAttacker .. " spies conducted a Covert Operation against " .. sRumorLocation .. "."
		sWarnString = sWarnString .. " " .. sRumorReport
		sFadeString = "FIR: " .. sRumorAttacker .. " spies conduct Covert Operation"		-- against " .. sRumorLocation
	end

	return sWarnString, sFadeString, iX, iY, iReportQuality									-- Report quality is the # of specific items in the report (max = 5)

end


function ProcessFIR(pPlayer, iGameTurn, iCurrentEra)										-- Determines if a historical FIR Notification is required this turn - only for human player!
	-- There is a small chance of receiving a report about covert operation directed against another civ.  Chance is based on the civ capital's EspionagePosture
	-- This is why the EP is checked every turn for the human player - in case the EP improves (more buildings, units, spies, etc.)
	local iEspionagePosture = CalculateEspionagePosture(pPlayer, pPlayer:GetCapitalCity())	-- Get pPlayer's EP, i.e. how competent pPlayer's intel apparatus is
	local sWarnString = ""
	local sFadeString = ""
	local iReportQuality = 0
	local iX = -1
	local iY = -1	
	local pPlayerTeam = Teams[pPlayer:GetTeam()]	
	
	local iChanceToView = math.floor(iEspionagePosture / 3)									-- Original EP / 4 = chance to view foreign report; changed to EP/3, better chance of details (sman:20180215)
	if iChanceToView < 10 then iChanceToView = 10											-- Minimum chance to view is 2%; changed to 10% (sman:20180209)
	elseif iChanceToView > (12 + (iCurrentEra*3)) then iChanceToView = 12 + (iCurrentEra*3) end	-- Maximum "potential" chance to view is 8% + (Current Era*2); Information Era = 7 (22%); Changed to 12+(Era*2), or 15..33% (sman:20180209); potential, because if poor quality report, notification won't be sent

	if math.random(1, 100) < iChanceToView then												-- Can we see the report?
		for iIndex = #gAiMissionTable, 1, -1 do												-- Cycle through all missions in the table backwards - looking for first closed mission that isn't directed at pPlayer; missions are added in chronological scheduling turn order - later missions at bottom
			
			local iAttackingTeamID = Players[gAiMissionTable[iIndex][1]]:GetTeam()			-- Get Team ID for attacking civ [1]
			local iTargetTeamID = Players[gAiMissionTable[iIndex][7]]:GetTeam()				-- Get Team ID for the defending civ [7]
			
			--print("FIR. Mission open [2] = " .. gAiMissionTable[iIndex][2] .. "   Attacker: " .. gAiMissionTable[iIndex][1] .. "    Defender: " .. gAiMissionTable[iIndex][7])

			if (gAiMissionTable[iIndex][2] == NO) and (gAiMissionTable[iIndex][7] ~= pPlayer:GetID()) and	-- Is the mission OPEN [2]? We want CLOSED.  Is the target NOT a human/pPlayer [7]?  and...   Not testing if any civ involved IsAlive - this is a historical report...
					( pPlayerTeam:IsHasMet(iAttackingTeamID) or pPlayerTeam:IsHasMet(iTargetTeamID)	) then	-- Has pPlayer met either the attacker or defender?  If so, grab the most recent report										
								
				sWarnString, sFadeString, iX, iY, iReportQuality = BuildWarningString(pPlayer, iGameTurn, iIndex, iEspionagePosture, false)	-- Called function: the last argument="IsRUMINT" if true, it will generate a future/impending report (aka RUMINT); if false then past reporting (FIR)
				
				if iReportQuality >= 2 then													-- Message has to have 2 or more elements "correct" otherwise, abandon
					SendNotification(4, sWarnString, sFadeString, nil, iX, iY)				-- Issue Notification
					return																	-- Exit the loop early; Changed from 'break' (sman:20180219)
				end				
			end
		end																					-- If the loop terminates normally, we didn't find any CLOSED missions targeting non-human players
	end																						-- If no missions found, no notification sent, continue on with processing...
end


function ProcessRUMINT(pPlayer, iGameTurn, iCurrentEra)									-- Determines if a Warning Notification is required this turn - only for human player!
	-- RUMINTs are generated if there is a positive value in gAiMissionTable[iIndex][4], aka "Turn to Warn" - which is determined when the mission is scheduled.
	-- If [4] is positive (game turn) - then a Notification will be sent - the quality threshold is 1 for RUMINT vice 2 for FIR - so, it's possible human player WON'T get RUMINT or receive a very poor quality report
	local iEspionagePosture = CalculateEspionagePosture(pPlayer, pPlayer:GetCapitalCity())	-- Get pPlayer's EP, i.e. how competent pPlayer's intel apparatus is
	local sWarnString = ""
	local sFadeString = ""
	local iReportQuality = 0
	local iX = -1
	local iY = -1
	
	for iIndex = 1, #gAiMissionTable do														-- Cycle through all missions in the table		
		if (gAiMissionTable[iIndex][2] == YES) and (gAiMissionTable[iIndex][7] == pPlayer:GetID()) and	-- [2] = Open mission, [7] = target human
				(gAiMissionTable[iIndex][4] == iGameTurn) and ( Players[gAiMissionTable[iIndex][1]]:IsAlive() ) then	-- [4] = warn turn = game turn, [1] = Attacker - don't report if attacker is gone...
			
			sWarnString, sFadeString, iX, iY, iReportQuality = BuildWarningString(pPlayer, iGameTurn, iIndex, iEspionagePosture, true)	-- last argument="Warning" if true, then future/impending report (aka RUMINT); if false then past reporting (FIR)
			
			--print("RUMINT. Mission No.: " .. iIndex .. "    Attacking Civ: " .. gAiMissionTable[iIndex][1] .. "    Quality: " .. iReportQuality .. "    Report: " .. sWarnString)

			if iReportQuality >= 1 then														-- Message has to have 1 or more elements "correct" otherwise, abandon
				SendNotification(4, sWarnString, sFadeString, nil, iX, iY)					-- Issue Notification
			end
		end
	end
end
